#!/bin/bash
#
# Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
#
# This software product is a proprietary product of NVIDIA CORPORATION &
# AFFILIATES (the "Company") and all right, title, and interest in and to the
# software product, including all associated intellectual property rights, are
# and shall remain exclusively with the Company.
#
# This software product is governed by the End User License Agreement
# provided with the software product.
#

# This script uses the dpacc tool (located in /opt/mellanox/doca/tools/dpacc) to compile DPA device
# code and build host stub lib.
# This script takes 3 arguments:
# arg1: Application name - The application's name, this name is used to create flexio_app struct
# arg2: Source file - Device source code
# arg3: Directory to install the DPA Device build, final output is <arg2>/<arg1>.a

# Input parameters
APP_NAME=$1
SOURCE_FILE=$2
BUILD_DIR=$3

# Tools location - DPACC, DPA compiler
DOCA_TOOLS="/opt/mellanox/doca/tools"
DPACC="${DOCA_TOOLS}/dpacc"
DPA_CLANG="${DOCA_TOOLS}/dpa-clang"

# CC flags
DEV_CC_FLAGS="-Wall,-Wextra,-Wpedantic,-Werror,-O0,-g,-DE_MODE_LE,-ffreestanding,-march=rv64imac,-mabi=lp64,-mno-relax,-mcmodel=medany,-nostdlib,-Wdouble-promotion"
DEV_INC_DIR="-I/opt/mellanox/flexio/include"
DEV_FULL_CC_FLAGS="${DEV_CC_FLAGS},${DEV_INC_DIR}"

# LD flags
DEV_LD_FLAGS="-march=rv64imac,-mabi=lp64,-fPIE,-flto,-nostdlib,-Wl,-Wl,-lclang_rt.builtins-riscv64,-z,nognustack,-z,norelro,'\"'-Wl,-uflexio_hw_rpc'\"','\"'-Wl,-urpc_print_cfg'\"','\"'-Wl,-urpc_print_close'\"'"
DEV_LD_LIBS="-lflexio_dev,-lflexio-libc,-lflexio_os"
DEV_LIB_DIR="-L/opt/mellanox/flexio/lib"
DEV_FULL_LD_FLAGS="${DEV_LD_FLAGS},${DEV_LIB_DIR},${DEV_LD_LIBS}"

# Host flags
HOST_INC_DIR="-I/opt/mellanox/flexio/include/libflexio"

# Compile the DPA (kernel) device source code using the DPACC
${DPACC} ${SOURCE_FILE} -o "${BUILD_DIR}/${APP_NAME}.a" \
        -hostcc=gcc \
        --devicecc=${DPA_CLANG} \
        --devicecc-options=${DEV_FULL_CC_FLAGS} \
        --devicelink-options=${DEV_FULL_LD_FLAGS} \
	-hostcc-options=${HOST_INC_DIR} \
	--app-name=${APP_NAME}
